/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Excute application -- runs popen for you
 *
 * Copyright (C) 2005 Matt O'Gorman.  Rights assigned to Digium.
 * popen info - Runs a command and returns what command returns.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/options.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>

static char *tdesc = "Stores output of command into a variable";

static char *app_execute = "Execute";

static char *execute_synopsis = "Execute(varname=program,lines)";

static char *execute_descrip =
"Execute(varname=program,lines)\n"
"  Varname   - Variable you want to copy to\n"
"  Program - The name of the program to run, if c flag not called\n"
"  Length - Max length of variable.  if not set is infinite\n"
;

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static char * execute_popen(int len, char *prog)
{
    FILE *ast_command;
    char line[160];
    int i=1;
    int flag=0;
    char *output=NULL;
    ast_command  = popen(prog, "r");
    
    while(fgets(line,sizeof line,ast_command)&&(i < len||len==0)){
		if(flag==0){
			output=(char *)malloc((strlen(line)+1)*sizeof(char));
			strcpy(output,line);
			flag=1;
		}
		else{ 
        	if (output){ 
        	    output = (char *) realloc(output, (strlen(output)+strlen(line)+1)*sizeof(char));
        	    strcat(output,line);
        	    i++;
        	}
        	else
				ast_log(LOG_ERROR, "Out of memory\n");
    	}
    }
	
    pclose(ast_command);
    return output;
}

static int execute_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	struct localuser *u;
	char *s, *varname=NULL, *program=NULL, *length=NULL, *returnvar=NULL;
	int arglen;
	int len=0;
	
	LOCAL_USER_ADD(u);
	
	/* check my data */
	arglen = strlen(data);
	s = alloca(arglen + 1);
	if(!s){
		ast_log(LOG_DEBUG,"Memory allocation failed\n");
		return 0;
	}
	
	memcpy (s,data,arglen +1);

	if (strchr(s,'=')){
		varname= strsep(&s, "=");
		if (strchr(s, '"')){
			strsep(&s,"\"");
			if(strchr(s,'"')){
				program=strsep(&s,"\"");
				if(strchr(s,'|')){	
					strsep(&s,"|");
					length= strsep(&s, "\0");
				}
			}
			else
				ast_log(LOG_DEBUG,"Syntax error in argument\n");

		}
		else if (strchr(s, '|')){
			program= strsep(&s, "|");
			length= strsep(&s, "\0");
		}
		else{
			program= strsep(&s,"\0");
		}
		
		if(!varname||!program){
			ast_log(LOG_DEBUG,"Syntax error in argument\n");
			return 0;
		}
		
		else{
			if (length){
				if(!sscanf(length,"%d",&len))
					ast_log(LOG_ERROR, "Length is not a number, set length to max");
			}
			returnvar=execute_popen(len,program);
			pbx_builtin_setvar_helper(chan,varname,returnvar);
		}
	}
	
	LOCAL_USER_REMOVE(u);
	return res;
}


int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app_execute);
}

int load_module(void)
{
	return ast_register_application(app_execute, execute_exec, execute_synopsis, execute_descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
